/// <reference types="vite/client" />

declare module '*.vue' {
    import type { DefineComponent } from 'vue'
    const component: DefineComponent<{}, {}, any>
    export default component
}

/** 系统配置参数类型（在main中可以初始化的值） */
declare type SystemParametersType = {
    /** DeBug 开关 */
    debug?: boolean

    /** 展开搜索表单 */
    IssearchForm?: boolean
    /** 表单标签宽度 */
    labelWidth?: number

    /** 表格展开行是否开启 */
    expand?: boolean
    /** 展开行 列数量 */
    expandColumn?: number

    /** 操作菜单宽度 */
    menuWidth?: number
    /** 操作栏对齐位置 */
    menuAlign?: 'left' | 'center' | 'right'

    /** 对话框宽度 */
    dialogWidth?: number
    /** 对话框页脚按钮 */
    editFooterButton?: boolean

    /** 表格分页大小 */
    pageSizes?: number[]
    /** 分页参数 */
    Pagination?: {
        /* 当前页数 */
        current?: number
        /* 每页条数 */
        pageSize?: number
        /* 总条数 */
        total?: number
    }
    /** 分页组件布局，子组件名用逗号分隔 */
    PaginationLayout?: string
    /** 当分页只有一页时，隐藏分页组件 */
    hideOnSinglePage?: boolean

    /** 数据加载完成后是否立即查询 */
    queryAfterMount?: boolean
    /** 表格的多选框是否开启 */
    openMultipleSelection?: boolean
    /** 表格索引是否开启 */
    openIndex?: boolean
    /** 点击 model 关闭对话框 */
    destroyOnClosModal?: boolean

    /** api 接口name名称 */
    apiName?: ApiNameType
    /** 页面节点权限名称 */
    permission?: ApiNameType
    /** 拥有的权限 */
    permissions?: string[]
}

// 钩子对象约束
interface HooksConfigType {
    /** 钩子Debug */
    debug?: boolean
    /**提交之前 */
    beforeSubmitHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 启用/禁用之前 */
    beforedisabledHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 启用/禁用之后 */
    afterdisabledHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 新增之前 */
    beforeAddHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 编辑之前 */
    beforeEditHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 查询之前 */
    beforeQueryHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 查询之后 */
    afterQueryHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 删除之前 */
    beforeDeteleHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 删除之后 */
    afterDeteleHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 表格懒加载之前 */
    beforeTableLazyHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
}

/** Ref 暴露参数 */
declare type CrudStateType = {
    /** 分类原数据 */
    ElementArrs?: ElementArrsType
    /** 查询参数 */
    searchModel?: { [x: string]: any }
    searchFormValue?: { [x: string]: any }
    /** 编辑参数 */
    editModel?: { [x: string]: any }
    editFormValue?: { [x: string]: any }
    /** 上传文件暂存 */
    uploadFiles?: any[]
    /** 加载状态 */
    tableLoading?: boolean
    /** 分页状态 */
    Pagination?: any

    Mode: 'add' | 'edit'

    /** 查询请求 */
    queryRecord?: () => void
}

/** 数据源配置类型 */
declare type SourceOptType = {
    /** 搜索表单参数 并 排序 */
    filter: string[]
    /** 编辑表单参数 并 排序 */
    editor: string[]
    /** 表格列参数 并 排序 */
    column: string[]

    /** 具体参数 */
    columns: columnsType

    options: SystemParametersType
    // options: {
    //     /** 当前页面名称 */
    //     id?: string
    //     /** 操作菜单宽度 默认310 */
    //     menuWidth?: number
    //     /**接口 name 名称 */
    //     apiName?: ApiNameType
    //     /** 页面权限 */
    //     permission?: ApiNameType
    //     /** 分页大小 */
    //     pageSizes?: number[]
    //     /** 表单 label 宽度 */
    //     labelWidth?: number
    //     /** 搜索表单是否折叠 */
    //     IssearchForm?: boolean
    //     hideRefresh?: boolean
    //     hidAddSubmitBut?: boolean
    //     showActionBar?: boolean
    //     menuAlign?: string
    // }
}
/** 表单参数配置项 */
declare type columnsType = {
    [x: string]: columnsItemType
}
/** 表单参数配置项 */
declare type columnsItemType = {
    /** 渲染表单组件类型 */
    type: 'el-input' | 'el-select' | 'el-input-number' | 'el-date-picker' | 'el-time-picker' | 'el-cascader' | 'el-upload'
    /** 标签名 */
    label: string
    /** 在展开行中隐藏 */
    hidden?: boolean
    /** 校验规则 */
    rules?: any
    /** 在编辑表单中所占的列数  */
    span?: {
        xs?: number
        sm?: number
        md?: number
        lg?: number
        span?: number
    }
    /** 表格列对齐方式 */
    align?: string
    /** 搜索表单默认值 */
    searchDefaultValue?: any
    /** 新增表单默认值 */
    addDefaultValue?: any
    /** 新增/编辑模式 显示状态 */
    disabled?: (CrudState: any, mode: string) => boolean
    /** 新增/编辑模式 显示状态 */
    visible?: (CrudState: any, mode: string) => boolean
    /** 是否为范围选择器 */
    range?: boolean
    /** 文件上传时图片预览地址 */
    fileList?: { name: string; url: string }[]
    /** 表单组件配置参数 */
    formProps?: any
    /** 表格配置参数 */
    tableProps?: any
    /** 表格列组件化插槽配置 */
    columnSlot?: any
}

/** 接口 name 默认名称 */
declare type ApiNameType = {
    /** 查询 */
    query?: string
    /** 新增 */
    add?: string
    /** 编辑 */
    edit?: string
    /** 单条删除 */
    delete?: string
    /** 标记删除 [启用 / 禁用] */
    deleteFlag?: string
    /** 多选删除 */
    remove?: string
}

/** 数据监听参数 */
declare type watchDataType = {
    id: string
    data: {
        url: string
        name: string
        data: any
        key: string
        label: string | string[]
        value: string
    }[]
}

/** 对 columns 数据拆分*/
interface ElementArrsType {
    filter: columnsType
    editor: columnsType
    column: columnsType
}
